@@ -32,7 +32,7 @@ module Bundler
       @base_dg = Molinillo::DependencyGraph.new
       @base.each do |ls|
         dep = Dependency.new(ls.name, ls.version)
-        @base_dg.add_vertex(ls.name, DepProxy.new(dep, ls.platform), true)
+        @base_dg.add_vertex(ls.name, DepProxy.get_proxy(dep, ls.platform), true)
       end
       additional_base_requirements.each {|d| @base_dg.add_vertex(d.name, d) }
       @platforms = platforms
@@ -75,7 +75,7 @@ module Bundler
       return unless debug?
       debug_info = yield
       debug_info = debug_info.inspect unless debug_info.is_a?(String)
-      puts debug_info.split("\n").map {|s| "BUNDLER: " + "  " * depth + s }
+      puts debug_info.split("\n").map {|s| depth == 0 ? "BUNDLER: #{s}" : "BUNDLER(#{depth}): #{s}" }
     end
 
     def debug?
@@ -155,13 +155,20 @@ module Bundler
         search.each do |sg|
           next unless sg.for?(platform)
           sg_all_platforms = sg.copy_for(self.class.sort_platforms(@platforms).reverse)
+          next unless sg_all_platforms
+
           selected_sgs << sg_all_platforms
 
           next if sg_all_platforms.activated_platforms == [Gem::Platform::RUBY]
           # Add a spec group for "non platform specific spec" as the fallback
           # spec group.
           sg_ruby = sg.copy_for([Gem::Platform::RUBY])
-          selected_sgs.insert(-2, sg_ruby) if sg_ruby
+          next unless sg_ruby
+
+          sg_ruby_deps = sg_ruby.dependencies_for_activated_platforms.map(&:dep)
+          sg_all_platforms_deps = sg_all_platforms.dependencies_for_activated_platforms.map(&:dep)
+
+          selected_sgs.insert(-2, sg_ruby) if sg_ruby_deps != sg_all_platforms_deps
         end
         selected_sgs
       end
